from ovp_core import models
from ovp_core import validators
from rest_framework import serializers
from ovp_uploads.serializers import UploadedImageSerializer

class FullCauseSerializer(serializers.ModelSerializer):
  image = UploadedImageSerializer()
  class Meta:
    fields = ['id', 'name', 'image']
    model = models.Cause

class CauseSerializer(serializers.ModelSerializer):
	class Meta:
		fields = ['id', 'name']
		model = models.Cause

class CauseAssociationSerializer(serializers.ModelSerializer):
  id = serializers.IntegerField()
  name = serializers.CharField(read_only=True)

  class Meta:
    fields = ['id', 'name']
    model = models.Cause
    validators = [validators.cause_exist]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-05-23 17:33
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_core', '0008_availability'),
    ]

    operations = [
        migrations.AddField(
            model_name='lead',
            name='date',
            field=models.DateTimeField(auto_now_add=True, null=True, verbose_name='Date'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-06-20 18:52
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_core', '0011_simpleaddress'),
    ]

    operations = [
        migrations.AddField(
            model_name='simpleaddress',
            name='supplement',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
    ]



import csv
from datetime import datetime

from django.http import HttpResponse


def Response(rows, filename=None, sheet_name=None):
  if filename is None:
    filename = "data-export-{}.csv".format(datetime.now().strftime('%Y-%m-%d_%H%M%S'))
  response = HttpResponse(content_type='text/csv')
  response['Content-Disposition'] = 'attachment; filename="{}"'.format(filename)

  csv_writer = csv.writer(response)
  for row in rows:
    csv_writer.writerow(row)

  return response


from rest_framework import serializers

from ovp_core.serializers import GoogleAddressSerializer

from ovp_core.models import Skill
from ovp_core.models import Cause

def address_validate(address):
  address_sr = GoogleAddressSerializer(data=address)
  address_sr.is_valid(raise_exception=True)

def skill_exist(v): # pragma: no cover
  try:
    Skill.objects.get(id=v['id'])
  except Skill.DoesNotExist:
    raise serializers.ValidationError({'id': "Skill with 'id' {} does not exist.".format(v['id'])})

def cause_exist(v): # pragma: no cover
  try:
    Cause.objects.get(id=v['id'])
  except Cause.DoesNotExist:
    raise serializers.ValidationError({'id': "Cause with 'id' {} does not exist.".format(v['id'])})



from ovp_core import models
from rest_framework import serializers

class SimpleAddressSerializer(serializers.ModelSerializer):
  class Meta:
    model = models.SimpleAddress
    fields = ['street', 'number', 'neighbourhood', 'city', 'state', 'zipcode', 'country', 'supplement']



from ovp_core import validators
from ovp_core.models import Availability
from rest_framework import serializers


class AvailabilitySerializer(serializers.ModelSerializer):
  def to_representation(self, instance):
    return { 'weekday': instance.weekday, 'period': instance.period }

  def validate(self, data):
    if not 'period_index' in data:
      data['period_index'] = Availability.compose_period_index_for(data.get('weekday', 0), data.get('period', 0))
    return super().validate(data)

  class Meta:
    fields = ['weekday', 'period']
    model = Availability




from django.test import TestCase
from django.test.utils import override_settings

from ovp_core.helpers import get_address_model
from ovp_core.models import SimpleAddress, GoogleAddress

class GetAddressModelHelperTestCase(TestCase):
  def test_default_model(self):
    """Assert GoogleAddress is the default address model"""
    model = get_address_model()
    self.assertTrue(model == GoogleAddress)

  @override_settings(OVP_CORE={'ADDRESS_MODEL': 'ovp_core.models.SimpleAddress'})
  def test_setting(self):
    """Assert it's possible to modify the model by changing the setting"""
    model = get_address_model()
    self.assertTrue(model == SimpleAddress)



from django.conf import settings
from django.utils.translation import ugettext as _
import importlib

def get_settings(string="OVP_CORE"):
  return getattr(settings, string, {})


def import_from_string(val):
  try:
    # Nod to tastypie's use of importlib.
    parts = val.split('.')
    module_path, class_name = '.'.join(parts[:-1]), parts[-1]
    module = importlib.import_module(module_path)
    return getattr(module, class_name)
  except ImportError as e:
    msg = "Could not import '%s' for setting. %s: %s." % (val, e.__class__.__name__, e)
    raise ImportError(msg)


def is_email_enabled(email):
  """ Emails are activated by default. Returns false
      if an email has been disabled in settings.py
  """
  s = get_settings(string="OVP_EMAILS")
  email_settings = s.get(email, {})

  enabled = True
  if email_settings.get("disabled", False):
    enabled = False

  return enabled


def get_email_subject(email, default):
  """ Allows for email subject overriding from settings.py  """
  s = get_settings(string="OVP_EMAILS")
  email_settings = s.get(email, {})

  title = email_settings.get("subject", default)

  return _(title)


def get_address_model():
  """ Returns application address model

  The address model can be modified by setting OVP_CORE.ADDRESS_MODEL.

  Returns:
    class: Address model class

    The default model returned is ovp_core.models.GoogleAddress

  """
  model_name = get_settings().get("ADDRESS_MODEL", "ovp_core.models.GoogleAddress")
  return import_from_string(model_name)


def get_address_serializers():
  """ Return application address serializer tuple

  The tuple can be modified by setting OVP_CORE.ADDRESS_SERIALIZER_TUPLE.

  Returns:
    tuple: A tuple with 3 serializers

    The default tuple returned is (GoogleAddressSerializer, GoogleAddressLatLngSerializer, GoogleAddressCityStateSerializer).
    The first one is the default serializer, usually used to create and update addresses.
    The second extends the first one but also includes 'lat' and 'lng' field, used to create pins on maps.
    The third is a simplified serializer containing only field 'city_state'. This is used on search.

  """
  serializers = get_settings().get('ADDRESS_SERIALIZER_TUPLE', ('ovp_core.serializers.GoogleAddressSerializer', 'ovp_core.serializers.GoogleAddressLatLngSerializer', 'ovp_core.serializers.GoogleAddressCityStateSerializer'))
  return [import_from_string(s) for s in serializers]



from .address import GoogleAddress, AddressComponent, AddressComponentType, GoogleRegion, SimpleAddress
from .cause import Cause
from .skill import Skill
from .lead import Lead
from .availability import Availability



from django.db import models

class SimpleAddress(models.Model):
  street = models.CharField(max_length=300, null=True, blank=True)
  number = models.CharField(max_length=10, null=True, blank=True) # May contain letters as well
  neighbourhood = models.CharField(max_length=100, null=True, blank=True)
  city = models.CharField(max_length=100, null=True, blank=True)
  state = models.CharField(max_length=100, null=True, blank=True)
  zipcode = models.CharField(max_length=20, null=True, blank=True)
  country = models.CharField(max_length=100, null=True, blank=True)
  supplement = models.CharField(max_length=100, null=True, blank=True)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-06-13 18:21
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_core', '0010_merge_20170609_1453'),
    ]

    operations = [
        migrations.CreateModel(
            name='SimpleAddress',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('street', models.CharField(blank=True, max_length=300, null=True)),
                ('number', models.CharField(blank=True, max_length=10, null=True)),
                ('neighbourhood', models.CharField(blank=True, max_length=100, null=True)),
                ('city', models.CharField(blank=True, max_length=100, null=True)),
                ('state', models.CharField(blank=True, max_length=100, null=True)),
                ('zipcode', models.CharField(blank=True, max_length=20, null=True)),
                ('country', models.CharField(blank=True, max_length=100, null=True)),
            ],
        ),
    ]



from io import BytesIO
from datetime import datetime

from pyexcel_xls import save_data

from django.http import HttpResponse


def Response(rows, filename=None,sheet_name="root"):
  if filename is None:
    filename = "data-export-{}.xls".format(datetime.now().strftime('%Y-%m-%d_%H%M%S'))

  xls_buffer = BytesIO()
  save_data(xls_buffer, {str(sheet_name): rows})

  response = HttpResponse(xls_buffer.getvalue(), content_type='application/vnd.ms-excel')
  response['Content-Disposition'] = 'attachment; filename="{}"'.format(filename)

  return response


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


availability_periods = [(0, 'Manhã'), (1, 'Tarde'), (2, 'Noite')]
availability_weekdays = [(0, 'Domingo'), (1, 'Segunda'), (2, 'Terça'), (3, 'Quarta'), (4, 'Quinta'), (5, 'Sexta'), (6, 'Sábado')]

def foward_func(apps, schema_editor):
    Availability = apps.get_model('ovp_core', 'Availability')
    for period in range(0, len(availability_periods)):
        for weekday in range(0, len(availability_weekdays)):
            index = weekday * len(availability_weekdays) + period
            Availability.objects.create(weekday=weekday, period=period, period_index=index)
    return True

def rewind_func(apps, schema_editor):
    Availability = apps.get_model('ovp_core', 'Availability')
    Availability.objects.all().delete()
    return True

class Migration(migrations.Migration):

    dependencies = [
        ('ovp_core', '0007_lead'),
    ]

    operations = [
        migrations.CreateModel(
            name='Availability',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('weekday', models.PositiveSmallIntegerField(choices=availability_weekdays, default=0, verbose_name='Weekday')),
                ('period', models.PositiveSmallIntegerField(choices=availability_periods, default=0, verbose_name='Day period')),
                ('period_index', models.PositiveSmallIntegerField(db_index=True)),
            ],
        ),
        migrations.RunPython(foward_func, rewind_func)
    ]



from django.db import models
from django.utils.translation import ugettext_lazy as _


availability_periods = (
  (0, 'Manhã'),
  (1, 'Tarde'),
  (2, 'Noite')
)
availability_weekdays = (
  (0, 'Domingo'),
  (1, 'Segunda'),
  (2, 'Terça'),
  (3, 'Quarta'),
  (4, 'Quinta'),
  (5, 'Sexta'),
  (6, 'Sábado')
)

class Availability(models.Model):
  weekday = models.PositiveSmallIntegerField(_('Weekday'), choices=availability_weekdays, default=0)
  period = models.PositiveSmallIntegerField(_('Day period'), choices=availability_periods, default=0)
  period_index = models.PositiveSmallIntegerField(db_index=True)

  def save(self, *args, **kwargs):
    if self.pk is None:
      self.period_index = __class__.compose_period_index_for(self.weekday, self.period)

  def __str__(self):
    return "{} de {}".format(self.get_weekday_display(), self.get_period_display())

  @staticmethod
  def compose_period_index_for(weekday, period):
    return int(weekday) * len(availability_weekdays) + int(period)

  @staticmethod
  def decompose_period_index(index):
    weekday_count = len(availability_weekdays)
    return int(index / weekday_count), index % weekday_count


from ovp_core import models
from ovp_core import validators
from rest_framework import serializers


class SkillSerializer(serializers.ModelSerializer):
  class Meta:
    fields = ['id', 'name']
    model = models.Skill

class SkillAssociationSerializer(serializers.ModelSerializer):
  id = serializers.IntegerField()
  name = serializers.CharField(read_only=True)

  class Meta:
    fields = ['id', 'name']
    model = models.Skill
    validators = [validators.skill_exist]
